-- SlashCommand by Altariel

-- TODO
-- * Bug fixes!
-- * /slashalias
local VERSION = "0.3.2";
local RELEASEDATE = "June 4, 2006";

SlashCommandDetails = {
	name = "SlashCommand",
	description = "Enables users to create custom slash-commands",
	version = VERSION,
	releaseDate = RELEASEDATE,
	author = "Altariel",
	email = "altariel@subpop.net",
	website = "http://subpop.net/wow",
	category = MYADDONS_CATEGORY_CHAT,
	frame = "SlashCommandFrame",
	--optionsframe = "HelloWorldOptionsFrame"
};

SlashCommandHelp = {};
SlashCommandHelp[1] = "Usage:\n\n  * /slashadd cmdname text\n  * /slashdel cmdname\n  * /slashreplace cmdname text\n  * /slashprint\n  * /slashalias: Not yet implemented\n\nMultiple commands may be specified on the same line; just make sure that each command begins with a forward slash (/), and ends with a semicolon (;), even the last command in your list.\nCommand arguments may be passed to each slash-command too, specify them on the command line, and they'll be replaced in order. In your /slashadd command text, you may use %1, %2, and so on, which will be replaced. You may also use %t to get the name of your target.";

function SlashCommand_OnLoad()
	SLASH_SLASHADD1 = "/slashadd";
	SlashCmdList["SLASHADD"] = SlashCommand_SlashAddCmd;

	SLASH_SLASHDEL1 = "/slashdel";
	SLASH_SLASHDEL2 = "/slashrm";
	SlashCmdList["SLASHDEL"] = SlashCommand_SlashDelCmd;

	SLASH_SLASHREPLACE1 = "/slashreplace";
	SlashCmdList["SLASHREPLACE"] = SlashCommand_SlashReplaceCmd;

	SLASH_SLASHALIAS1 = "/slashalias";
	SlashCmdList["SLASHALIAS"] = SlashCommand_SlashAliasCmd;

	SLASH_SLASHPRINT1 = "/slashprint";
	SLASH_SLASHPRINT2 = "/slashls";
	SlashCmdList["SLASHPRINT"] = SlashCommand_PrintCommands;
	
	SLASH_SLASHEDIT1 = "/slashedit";
	SlashCmdList["SLASHEDIT"] = SlashCommand_SlashEditCmd;

	this:RegisterEvent("VARIABLES_LOADED");

	if ( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("SlashCommand " .. VERSION .. " loaded.");
	end
end

function SlashCommand_OnEvent()
	if(event == "VARIABLES_LOADED") then
		if(not SlashCommandCommands) then
			SlashCommandCommands = {};
		end

		if(getn(SlashCommandCommands) > 0) then
			for i = 1, getn(SlashCommandCommands) do
				SlashCommand_LoadSlash(SlashCommandCommands[i].title);
			end
		end

		-- Register the addon in myAddOns
		if(myAddOnsFrame_Register) then
			myAddOnsFrame_Register(SlashCommandDetails, SlashCommandHelp);
		end
		
	end
end	

function SlashCommand_SlashAddCmd(args)
	local a,b = string.find(args,"^%w+%s");
	if(a and b) then
		local cmd = string.sub(args,a,b-1);
		local text = string.sub(args,b+1,string.len(args));

		local ret = SlashCommand_AddSlash(cmd, text);

		if(ret) then
			DEFAULT_CHAT_FRAME:AddMessage("SlashCommand: Added command '" .. cmd .. "' successfully");
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("SlashCommand: Usage: /slashadd name text");
	end

	for i = 1, getn(SlashCommandCommands) do
		SlashCommand_LoadSlash(SlashCommandCommands[i].title);
	end
end

function SlashCommand_SlashEditCmd(args)
	SlashCommand_EditSlash(args);
end

function SlashCommand_SlashDelCmd(args)
	SlashCommand_DelSlash(args);
end

function SlashCommand_SlashReplaceCmd(args)
	local a,b = string.find(args,"^%w+%s");
	if(a and b) then
		local cmd = string.sub(args,a,b-1);
		local text = string.sub(args,b+1,string.len(args));

		local ret = SlashCommand_ReplaceSlash(cmd, text);

		if(ret) then
			DEFAULT_CHAT_FRAME:AddMessage("SlashCommand: Replaced command '" .. cmd .. "' successfully");
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("SlashCommand: Usage: /slashreplace name text");
	end
end

function SlashCommand_SlashAliasCmd(args)
end

function SlashCommand_AddSlash(command, text)
	local ret, err = SlashCommand_CheckDuplicates(command);
	if(ret) then
		DEFAULT_CHAT_FRAME:AddMessage("SlashCommand Error: " .. err);
		return false;
	end

	local num = getn(SlashCommandCommands);

	local tmp = {
		["title"] = command,
		["text"]  = text,
	}
	table.insert(SlashCommandCommands,tmp);
	SlashCommand_LoadSlash(command);

	if(getn(SlashCommandCommands) == num+1) then
		return true;
	else
		return false;
	end
end

function SlashCommand_LoadSlash(command)
	setglobal("SLASH_SC" .. string.upper(command) .. "1", "/" .. command);
	SlashCmdList["SC" .. string.upper(command)] = SlashCommand_ExeCommand;
end

function SlashCommand_UnloadSlash(command)
	setglobal("SLASH_SC" .. string.upper(command) .. "1", nil);
	SlashCmdList["SC" .. string.upper(command)] = nil;
end

function SlashCommand_DelSlash(command)
	for i = 1, getn(SlashCommandCommands) do
		if(SlashCommandCommands[i].title == command) then
			table.remove(SlashCommandCommands, i);
			SlashCommand_UnloadSlash(command);

			DEFAULT_CHAT_FRAME:AddMessage("SlashCommand: Removed command '" .. command .. "' successfully");
			return;
		end
	end
end

function SlashCommand_Clear()
	SlashCommandCommands = {};
end

function SlashCommand_ReplaceSlash(command, text)
	local num = getn(SlashCommandCommands);

	for i = 1, num do
		if(SlashCommandCommands[i].title == command) then
			SlashCommandCommands[i] = {
				["title"] = command,
				["text"]  = text,
			}
		end
	end
	SlashCommand_LoadSlash(command);

	return true;
end

function SlashCommand_EditSlash(command)
	SlashCommandEditTitle:SetText(command);
	
	for i = 1, getn(SlashCommandCommands) do
		if(SlashCommandCommands[i].title == command) then
			SlashCommandEditScrollFrameText:SetText(SlashCommandCommands[i].text);
		end
	end
	
	SlashCommandEdit:Show();
end

function SlashCommand_SaveEdit()
	for i = 1, getn(SlashCommandCommands) do
		if(SlashCommandCommands[i].title == SlashCommandEditTitle:GetText()) then
			SlashCommandCommands[i].text = SlashCommandEditScrollFrameText:GetText();
		end
	end
end

function SlashCommand_ExeCommand(msg)
	local cmd = this:GetText();
	local args = {};
	local j = 0;

	local a,b = string.find(cmd,"%w+");
	cmd = string.sub(cmd,a,b);
	
	for w in string.gfind(msg, "[%w%p%c]+") do
		j = j + 1;
		args[j] = w;
	end

	for i = 1, getn(SlashCommandCommands) do
		if( SlashCommandCommands[i].title == cmd) then
			--DEFAULT_CHAT_FRAME:AddMessage("Text before string replacement: " .. SlashCommandCommands[i].text);
			local text = SlashCommandCommands[i].text;
			for index,arg in args do
				text = string.gsub(text, "%%" .. index, arg);
			end
			if(UnitName("target")) then
				text = string.gsub(text, "%%t",UnitName("target"));
			end
			--DEFAULT_CHAT_FRAME:AddMessage("Text to be executed is: " .. text);

			local cmds = {};
			for c in string.gfind(text, "(/.-);") do
				table.insert(cmds,c);
			end
			
			if(getn(cmds) > 1) then
				for i = 1, getn(cmds) do
					this:SetText(cmds[i]);
					ChatEdit_OnEnterPressed();
				end
			else		
				this:SetText(text);
				ChatEdit_OnEnterPressed();
			end
		end
	end
end

function SlashCommand_CheckDuplicates(command)
	local cmds = getn(SlashCommandCommands);

	if(getn(SlashCommandCommands) > 0) then
		for i = 1, getn(SlashCommandCommands) do
			if(SlashCmdList["SC" .. string.upper(command)]) then
				return true,"Command '" .. command .. "' already exists";
			end
		end
	end
	return false;
end

function SlashCommand_PrintCommands()
	DEFAULT_CHAT_FRAME:AddMessage("SlashCommand: Current commands (total " .. getn(SlashCommandCommands) .. "):");
	for i = 1, getn(SlashCommandCommands) do
		DEFAULT_CHAT_FRAME:AddMessage(tostring(i) .. ": " .. SlashCommandCommands[i].title .. " => " .. SlashCommandCommands[i].text);
	end
end
